import '../lib/pegdart.dart';

class Lisp extends Grammar {
  
  @override
  initialize() {
    
    // Whitespace.
    var ws = r('\\s').many.any.ignore;
    
    // Integers.
    var integer = r("\\d").many[#digits] >> (s) => s[#digits].join('');
    
    // Have to be careful to not include punctuations marks ('(', ')'), whitespace, and digits.
    var symbol = (-r('\\s|\\d|\\)|\\(') + wildcard).many[#syms] >> (s) => s[#syms].expand((x) => x).join('');
    
    // Atoms.
    var atom = symbol | integer;
    
    // This is where the recursion comes in because a list can include within it another expression.
    var expression = ref(#list) | atom;
    
    // We also have quoting.
    var quotedExpression = (m("'") + expression[#expr]) >> (s) => [#quote, s[#expr]];
    
    // An expression can be either one.
    var generalExpression = quotedExpression | expression;
        
    // Sequencing adds an extra layer of nesting and we remove it here
    var wsExpression = (ws + generalExpression[#expr] + ws) >> (s) => s[#expr];
    
    // Exactly what you'd expect.
    var emptyList = m('()') >> (s) => [];
    
    // Non-empty lists are nested sequences of expressions.
    var nonEmptyList = (m('(') + cut + wsExpression.many[#elements] + m(')')) >> (s) => s[#elements];
    
    // The rule definition for the list. It is a rule because we need to recursively refer back to it
    // and we can't do that with simple variable assignment.
    rule(#list, emptyList | nonEmptyList);
    
    // Also have to tell the grammar which rule to start parsing with.
    rule(#start, wsExpression);
    
  }
  
}

// Simple demo.
main() {
  var grammar = new Lisp()..initialize();
  var result = grammar.parse("(123 '(123) '())");
  print(result);
}