part of pegdart;

class JumpCutException implements Exception {
  final String message;
  const JumpCutException(this.message);
}

class MisplacedCutException implements Exception {
  final String message;
  const MisplacedCutException(this.message);
}

class InputContext {
  
  final dynamic indexable;
  final Map<Symbol, PEGExpr> rules;
  
  int currentPosition = 0, cutPosition = -1;
  
  InputContext(this.indexable, this.rules);
  
  /* Put the pointer at the given position unless there are cuts in the way. We can not jump behind cuts. */
  int jumpTo(int jumpPosition) {
    if (jumpPosition < cutPosition) {
      throw new JumpCutException("Can not jump behind cut position: cut = ${cutPosition}, jump = ${jumpPosition}.");
    }
    return currentPosition = jumpPosition;
  }
  
  /* Returns null in case of any failures. */
  dynamic get currentChar {
    try {
      return indexable[currentPosition];
    } catch (exception) {
      return null;
    }
  }
  
  /* Returns null in case of any failures. */
  dynamic advancePosition() {
    try {
      return indexable[++currentPosition];
    } catch (excpetion) {
      return null;
    }
  }
  
  /* Places a cut at the current position so that any kind of backtracking will not be able go further back than
   * this cut position. Comes in handy when trying to figure out if there is unnecessary backtracking in the grammar. */
  void cut() {
    if (currentPosition >= cutPosition) {
      cutPosition = currentPosition;
    } else {
      throw new MisplacedCutException("Misplaced cut: current position = ${currentPosition}, existing cut = ${cutPosition}.");
    }
  }
  
}