import '../lib/pegdart.dart';
import 'package:unittest/unittest.dart';

class TestGrammar extends Grammar {
  initialize() {
    rule(#start, m('a').ignore.many + m('b').many);
  }
}

class AnotherTestGrammar extends Grammar {
  initialize() {
    rule(#start, (m('a') + m('b')).ignore.many);
  }
}

class YetAnotherTestGrammar extends Grammar {
  initialize() {
    rule(#start, (m('ab').ignore.many));
  }
}

main() {
  
  List<dynamic> flatten(Iterable list) {
    return list.expand((x) => x).map((r) => r.match).toList();
  }

  test('Ignore all coupled ab sequences.', () {
    var grammar = new YetAnotherTestGrammar()..initialize();
    var result = grammar.parse('abab');
    expect(result, Ignore);
  });
  
  test('Ignore all ab sequences.', () {
    var grammar = new AnotherTestGrammar()..initialize();
    var result = grammar.parse('ababab');
    expect(result, Ignore);
  });
  
  test('Ignore all ab sequence failure.', () {
    var grammar = new AnotherTestGrammar()..initialize();
    var result = grammar.parse('ababa');
    expect(result, Ignore);
  });
  
  test('One a and one b.', () {
    var grammar = new TestGrammar()..initialize();
    var result = flatten(grammar.parse('ab'));
    expect(result, ['b']);
  });
  
  test('Many a and one b.', () {
    var grammar = new TestGrammar()..initialize();
    var result = flatten(grammar.parse('aaab'));
    expect(result, ['b']);
  });
  
  test('Many a and many b.', () {
    var grammar = new TestGrammar()..initialize();
    var result = flatten(grammar.parse('aaabbb'));
    expect(result, ['b', 'b', 'b']);
  });
  
  test('One a and many b.', () {
    var grammar = new TestGrammar()..initialize();
    var result = flatten(grammar.parse('abbb'));
    expect(result, ['b', 'b', 'b']);
  });
  
  test('One a and no b.', () {
    var grammar = new TestGrammar()..initialize();
    expect(grammar.parse('a'), null);
  });
  
  test('Empty string.', () {
    var grammar = new TestGrammar()..initialize();
    expect(grammar.parse(''), null);
  });
  
}